@@ -4,7 +4,9 @@ from .eval_hooks import DistEvalHook, EvalHook
 from .mesh_eval import compute_similarity_transform
 from .pose3d_eval import keypoint_mpjpe
 from .top_down_eval import (keypoint_auc, keypoint_epe, keypoint_pck_accuracy,
-                            keypoints_from_heatmaps, keypoints_from_regression,
+                            keypoints_from_heatmaps, keypoints_from_heatmaps3d,
+                            keypoints_from_regression,
+                            multilabel_classification_accuracy,
                             pose_pck_accuracy, post_dark_udp)
 
 __all__ = [
@@ -12,5 +14,6 @@ __all__ = [
     'keypoints_from_regression', 'keypoint_pck_accuracy', 'keypoint_auc',
     'keypoint_epe', 'get_group_preds', 'get_multi_stage_outputs',
     'aggregate_results', 'compute_similarity_transform', 'post_dark_udp',
-    'keypoint_mpjpe'
+    'keypoint_mpjpe', 'keypoints_from_heatmaps3d',
+    'multilabel_classification_accuracy'
 ]
