@@ -1,9 +1,10 @@
 from .bottom_up_aic import BottomUpAicDataset
 from .bottom_up_coco import BottomUpCocoDataset
+from .bottom_up_coco_wholebody import BottomUpCocoWholeBodyDataset
 from .bottom_up_crowdpose import BottomUpCrowdPoseDataset
 from .bottom_up_mhp import BottomUpMhpDataset
 
 __all__ = [
     'BottomUpCocoDataset', 'BottomUpCrowdPoseDataset', 'BottomUpMhpDataset',
-    'BottomUpAicDataset'
+    'BottomUpAicDataset', 'BottomUpCocoWholeBodyDataset'
 ]
