@@ -77,13 +77,13 @@ class FreiHandDataset(HandBaseDataset):
     def _get_db(self):
         """Load dataset."""
         gt_db = []
+        bbox_id = 0
+        num_joints = self.ann_info['num_joints']
         for img_id in self.img_ids:
-            num_joints = self.ann_info['num_joints']
 
             ann_ids = self.coco.getAnnIds(imgIds=img_id, iscrowd=False)
             objs = self.coco.loadAnns(ann_ids)
 
-            rec = []
             for obj in objs:
                 if max(obj['keypoints']) == 0:
                     continue
@@ -99,7 +99,7 @@ class FreiHandDataset(HandBaseDataset):
 
                 image_file = os.path.join(self.img_prefix,
                                           self.id2name[img_id])
-                rec.append({
+                gt_db.append({
                     'image_file': image_file,
                     'center': center,
                     'scale': scale,
@@ -108,9 +108,12 @@ class FreiHandDataset(HandBaseDataset):
                     'joints_3d_visible': joints_3d_visible,
                     'dataset': self.dataset_name,
                     'bbox': obj['bbox'],
-                    'bbox_score': 1
+                    'bbox_score': 1,
+                    'bbox_id': bbox_id
                 })
-            gt_db.extend(rec)
+                bbox_id = bbox_id + 1
+        gt_db = sorted(gt_db, key=lambda x: x['bbox_id'])
+
         return gt_db
 
     def evaluate(self, outputs, res_folder, metric='PCK', **kwargs):
@@ -125,13 +128,12 @@ class FreiHandDataset(HandBaseDataset):
 
         Args:
             outputs (list(preds, boxes, image_path, output_heatmap))
-                :preds (np.ndarray[1,K,3]): The first two dimensions are
+                :preds (np.ndarray[N,K,3]): The first two dimensions are
                     coordinates, score is the third dimension of the array.
-                :boxes (np.ndarray[1,6]): [center[0], center[1], scale[0]
+                :boxes (np.ndarray[N,6]): [center[0], center[1], scale[0]
                     , scale[1],area, score]
-                :image_path (list[str]): For example, [ 't', 'r', 'a', 'i',
-                    'n', 'i', 'n', 'g', '/', 'r', 'g', 'b', '/', '0', '0',
-                    '0', '3', '1', '4', '2', '6', '.', 'j', 'p', 'g']
+                :image_paths (list[str]): For example, ['training/rgb/
+                    00031426.jpg']
                 :output_heatmap (np.ndarray[N, K, H, W]): model outpus.
 
             res_folder (str): Path of directory to save the results.
@@ -150,19 +152,26 @@ class FreiHandDataset(HandBaseDataset):
         res_file = os.path.join(res_folder, 'result_keypoints.json')
 
         kpts = []
+        for output in outputs:
+            preds = output['preds']
+            boxes = output['boxes']
+            image_paths = output['image_paths']
+            bbox_ids = output['bbox_ids']
 
-        for preds, boxes, image_path, _ in outputs:
-            str_image_path = ''.join(image_path)
-            image_id = self.name2id[str_image_path[len(self.img_prefix):]]
+            batch_size = len(image_paths)
+            for i in range(batch_size):
+                image_id = self.name2id[image_paths[i][len(self.img_prefix):]]
 
-            kpts.append({
-                'keypoints': preds[0].tolist(),
-                'center': boxes[0][0:2].tolist(),
-                'scale': boxes[0][2:4].tolist(),
-                'area': float(boxes[0][4]),
-                'score': float(boxes[0][5]),
-                'image_id': image_id,
-            })
+                kpts.append({
+                    'keypoints': preds[i].tolist(),
+                    'center': boxes[i][0:2].tolist(),
+                    'scale': boxes[i][2:4].tolist(),
+                    'area': float(boxes[i][4]),
+                    'score': float(boxes[i][5]),
+                    'image_id': image_id,
+                    'bbox_id': bbox_ids[i]
+                })
+        kpts = self._sort_and_unique_bboxes(kpts)
 
         self._write_keypoint_results(kpts, res_file)
         info_str = self._report_metric(res_file, metrics)
