@@ -135,6 +135,7 @@ class InterHand2DDataset(HandBaseDataset):
         with open(self.joint_file, 'r') as f:
             joints = json.load(f)
         gt_db = []
+        bbox_id = 0
         for img_id in self.img_ids:
             num_joints = self.ann_info['num_joints']
 
@@ -173,7 +174,6 @@ class InterHand2DDataset(HandBaseDataset):
                 joint_valid[hand, :] *= joint_valid[hand][-1]
 
                 if np.sum(joint_valid[hand, :]) > 11:
-                    rec = []
                     joints_3d = np.zeros((num_joints, 3), dtype=np.float32)
                     joints_3d_visible = np.zeros((num_joints, 3),
                                                  dtype=np.float32)
@@ -196,7 +196,7 @@ class InterHand2DDataset(HandBaseDataset):
                     # use 1.5bbox as input
                     center, scale = self._xywh2cs(*bbox, 1.5)
 
-                    rec.append({
+                    gt_db.append({
                         'image_file': image_file,
                         'center': center,
                         'scale': scale,
@@ -205,9 +205,11 @@ class InterHand2DDataset(HandBaseDataset):
                         'joints_3d_visible': joints_3d_visible,
                         'dataset': self.dataset_name,
                         'bbox': bbox,
-                        'bbox_score': 1
+                        'bbox_score': 1,
+                        'bbox_id': bbox_id
                     })
-                    gt_db.extend(rec)
+                    bbox_id = bbox_id + 1
+        gt_db = sorted(gt_db, key=lambda x: x['bbox_id'])
 
         return gt_db
 
@@ -223,11 +225,11 @@ class InterHand2DDataset(HandBaseDataset):
 
         Args:
             outputs (list(preds, boxes, image_path, output_heatmap))
-                :preds (np.ndarray[1,K,3]): The first two dimensions are
+                :preds (np.ndarray[N,K,3]): The first two dimensions are
                     coordinates, score is the third dimension of the array.
-                :boxes (np.ndarray[1,6]): [center[0], center[1], scale[0]
+                :boxes (np.ndarray[N,6]): [center[0], center[1], scale[0]
                     , scale[1],area, score]
-                :image_path (list[str]): For example, ['C', 'a', 'p', 't',
+                :image_paths (list[str]): For example, ['C', 'a', 'p', 't',
                     'u', 'r', 'e', '1', '2', '/', '0', '3', '9', '0', '_',
                     'd', 'h', '_', 't', 'o', 'u', 'c', 'h', 'R', 'O', 'M',
                     '/', 'c', 'a', 'm', '4', '1', '0', '2', '0', '9', '/',
@@ -251,19 +253,26 @@ class InterHand2DDataset(HandBaseDataset):
         res_file = os.path.join(res_folder, 'result_keypoints.json')
 
         kpts = []
+        for output in outputs:
+            preds = output['preds']
+            boxes = output['boxes']
+            image_paths = output['image_paths']
+            bbox_ids = output['bbox_ids']
 
-        for preds, boxes, image_path, _ in outputs:
-            str_image_path = ''.join(image_path)
-            image_id = self.name2id[str_image_path[len(self.img_prefix):]]
+            batch_size = len(image_paths)
+            for i in range(batch_size):
+                image_id = self.name2id[image_paths[i][len(self.img_prefix):]]
 
-            kpts.append({
-                'keypoints': preds[0].tolist(),
-                'center': boxes[0][0:2].tolist(),
-                'scale': boxes[0][2:4].tolist(),
-                'area': float(boxes[0][4]),
-                'score': float(boxes[0][5]),
-                'image_id': image_id,
-            })
+                kpts.append({
+                    'keypoints': preds[i].tolist(),
+                    'center': boxes[i][0:2].tolist(),
+                    'scale': boxes[i][2:4].tolist(),
+                    'area': float(boxes[i][4]),
+                    'score': float(boxes[i][5]),
+                    'image_id': image_id,
+                    'bbox_id': bbox_ids[i]
+                })
+        kpts = self._sort_and_unique_bboxes(kpts)
 
         self._write_keypoint_results(kpts, res_file)
         info_str = self._report_metric(res_file, metrics)
