@@ -4,7 +4,7 @@ from collections import OrderedDict
 import json_tricks as json
 import numpy as np
 
-from mmpose.core.evaluation import compute_similarity_transform
+from mmpose.core.evaluation.mesh_eval import compute_similarity_transform
 from mmpose.datasets.builder import DATASETS
 from .mesh_base_dataset import MeshBaseDataset
 
