@@ -4,7 +4,7 @@ import numpy as np
 from mmpose.core.post_processing import (affine_transform, fliplr_joints,
                                          get_affine_transform, get_warp_matrix,
                                          warp_affine_joints)
-from mmpose.datasets.registry import PIPELINES
+from mmpose.datasets.builder import PIPELINES
 
 
 @PIPELINES.register_module()
@@ -12,8 +12,9 @@ class TopDownRandomFlip:
     """Data augmentation with random image flip.
 
     Required keys: 'img', 'joints_3d', 'joints_3d_visible', 'center' and
-    'ann_info'. Modifies key: 'img', 'joints_3d', 'joints_3d_visible' and
-    'center'.
+    'ann_info'.
+    Modifies key: 'img', 'joints_3d', 'joints_3d_visible', 'center' and
+    'flipped'.
 
     Args:
         flip (bool): Option to perform random flip.
@@ -30,9 +31,12 @@ class TopDownRandomFlip:
         joints_3d_visible = results['joints_3d_visible']
         center = results['center']
 
+        # A flag indicating whether the image is flipped,
+        # which can be used by child class.
+        flipped = False
         if np.random.rand() <= self.flip_prob:
+            flipped = True
             img = img[:, ::-1, :]
-
             joints_3d, joints_3d_visible = fliplr_joints(
                 joints_3d, joints_3d_visible, img.shape[1],
                 results['ann_info']['flip_pairs'])
@@ -42,6 +46,7 @@ class TopDownRandomFlip:
         results['joints_3d'] = joints_3d
         results['joints_3d_visible'] = joints_3d_visible
         results['center'] = center
+        results['flipped'] = flipped
 
         return results
 
@@ -594,7 +599,7 @@ class TopDownGenerateTarget:
 
 
 @PIPELINES.register_module()
-class TopDownGenerateTargetRegression():
+class TopDownGenerateTargetRegression:
     """Generate the target regression vector (coordinates).
 
     Required keys: 'joints_3d', 'joints_3d_visible', 'ann_info'. Modified keys:
@@ -645,3 +650,35 @@ class TopDownGenerateTargetRegression():
         results['target_weight'] = target_weight
 
         return results
+
+
+@PIPELINES.register_module()
+class TopDownRandomTranslation:
+    """Data augmentation with random translation.
+
+    Required key: 'scale' and 'center'. Modifies key: 'center'.
+
+    Notes:
+        bbox height: H
+        bbox width: W
+
+    Args:
+        trans_factor (float): Translating center to
+        ``[-trans_factor, trans_factor] * [W, H] + center``.
+        trans_prob (float): Probability of random translation.
+    """
+
+    def __init__(self, trans_factor=0.15, trans_prob=1.0):
+        self.trans_factor = trans_factor
+        self.trans_prob = trans_prob
+
+    def __call__(self, results):
+        """Perform data augmentation with random translation."""
+        center = results['center']
+        scale = results['scale']
+        if np.random.rand() <= self.trans_prob:
+            # reference bbox size is [200, 200] pixels
+            center += self.trans_factor * np.random.uniform(
+                -1, 1, size=2) * scale * 200
+        results['center'] = center
+        return results
